;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;             euler-delay, for general first order de's
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun euler-delay (f ynot start end tao &key axes title)
  "
;; Description:

	euler-delay (f ynot start end tao &key axes title)

For delay equations, you'll need a tao, and the length of the initial
list ynot will determine the number of steps in one tao unit.

;; Examples:
;;
;; This is our rabies delay equation:
;;
 (def alpha 1)
 (def mu 1)
 (def rho 5)
 (def beta 5)

 (def xknot mu) ;; set this at equilibrium
 (def yknot (/ mu 1000)) ;; this is our fraction of rabied animals introduced

 (def num 200)
 (def tao 1)
 (def ynot 1)
 (def tmin 0)
 (def tmax 1)
 (def start 0)
 (def end 4)

 ;; the blip is the length of time that the rabid animals spend initially
 (def blip 10)
 (def ynot 
     (transpose 
      (list 
       (repeat xknot (ceiling (* num (/  tao (- tmax tmin)))))
       (combine
	(repeat yknot blip)
	(repeat 0 (- (ceiling (* num (/  tao (- tmax tmin)))) blip))
	)
       )
      )
     )
 (defun f(x time xd) 
  (let (
	(xx (first x))
	(yy (second x))
	(xxd (first xd))
	(yyd (second xd))
	)
    (list
     (- (* alpha xx (- mu xx)) (* beta xxd yyd))
     (- (* beta xxd yyd) (* rho yy))
     )
    ))
 (def result (euler-delay #'f ynot start end tao :title \"Delay\"))
 (plot (iseq (length result)) (mapcar #'first result))
 (send my-plot :add-plot-item
      (list 'lines (iseq (length result)) (mapcar #'second result)))
 (plot (mapcar #'first result) (mapcar #'second result))
"
  (let* (
	 (hsize (/ tao (length ynot)))
	 (delay-index (length ynot))
	 (yvalues ynot)
	 (num (ceiling (/ (- end start) hsize)))
	 (timesteps (rseq start end num))
	 j
	 )
    ;; (printall num delay-index)
    (dotimes
     (i (- num delay-index))
     (setq j (1- (+ i delay-index)))
     ;; (printall "i" i "j" j)
     ;; (printall (last yvalues) hsize yvalues)
     (setq yvalues
	   (append yvalues 
		   (list
		    (+ (final yvalues)
		       (* hsize 
			  (funcall 
			   f 
			   (select yvalues j)
			   (select timesteps j)
			   (select yvalues i)
			   )
			  )
		       )
		    )
		   )
	   )
     )
    yvalues
    )
  )

(defun jen(&key (alpha 1)
		(mu 1)
		(rho 5)
		(beta 5)
		(xknot mu) ;; set this at equilibrium
		(num 800)
		(tao 1)
		(ynot 1)
		(tmin 0)
		(tmax 1)
		(start 0)
		(end 4)
		)
  (let* (
	 (len (ceiling (* num (/  tao (- tmax tmin)))))
	 (hsize (/ tao len))
	 (yknot (/ xknot 1000)) ;; this is our 
	 			;; fraction of rabied animals introduced
	 (ynot 
	 (transpose 
	  (list 
	   (repeat xknot len)
	   (mapcar (lambda(x) (* yknot (exp (- (* rho x)))))
		   (- (rseq tmin tmax len) tmin)
		   )
	   )
	  )
	 )
	 result
	)
    (defun f(x time xd) 
      (let (
	    (xx (first x))
	    (yy (second x))
	    (xxd (first xd))
	    (yyd (second xd))
	    )
	(list
	 (- (* alpha xx (- mu xx)) (* beta xxd yyd))
	 (- (* beta xxd yyd) (* rho yy))
	 )
	))
    (def result (euler-delay #'f ynot start end tao :title "Delay"))
    (plot (iseq (length result)) (mapcar #'first result))
    (send my-plot :add-plot-item
	  (list 'lines (iseq (length result)) (mapcar #'second result)))
    (plot (mapcar #'first result) (mapcar #'second result))
    (plot (mapcar #'second result))
    )
  )
(jen)
